
import java.awt.*;
import javax.swing.*;

import javax.media.j3d.*;
import javax.vecmath.*;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;

import java.awt.event.*;

public class HelloUniverse extends JFrame {

	private TransformGroup tg;
	private Vector3f position;
	
	public HelloUniverse(){
		super();
		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
		
		Canvas3D c = new Canvas3D(config);
		this.getContentPane().add(c);
		
		SimpleUniverse u = new SimpleUniverse(c);
		
		u.getViewingPlatform().setNominalViewingTransform();
		
		
		//Kommentieren Sie die jeweils gewnschte "BranchGroup bg = ..." Zeile
		//aus, um eine bestimmte Szene anzeigen zu lassen.
		
		
		//Erste Szene + Transformationen:
		//BranchGroup bg = buildColorCubeVersion();
		
		//Material + Licht:
		//BranchGroup bg = buildMaterialandLightVersion();
		
		//Animation:
		//BranchGroup bg = buildAnimationVersion();
		
		//Tastaturinteraktion:
		BranchGroup bg = buildKeyboardInteractionVersion(c);
		
		u.addBranchGraph(bg);
	}
	
	/**
	 * Erzeugt eine neue BranchGroup, die einen um 60 um die X-Achse
	 * gedrehten ColorCube enthlt, der sich an der Position (3,3,-10) befindet.
	 */
	private BranchGroup buildColorCubeVersion(){
		BranchGroup bg = new BranchGroup();
		ColorCube cube = new ColorCube(0.4);
		
		Transform3D verschiebung = new Transform3D();
		verschiebung.setTranslation(new Vector3f(3, 3, -10));
		
		Transform3D rotation = new Transform3D();
		rotation.rotX(Math.toRadians(60));
		verschiebung.mul(rotation);

		TransformGroup tg = new TransformGroup(verschiebung);
		tg.addChild(cube);
		bg.addChild(tg);

		return bg;
	}
	
	/**
	 * Erzeugt eine Kugel an der Position (3,3,-10) mit
	 * einem festgelegten Material und zwei Lichtquellen,
	 * die die Szene beleuchten.
	 */
	private BranchGroup buildMaterialandLightVersion(){
		BranchGroup bg = new BranchGroup();
		Sphere ball = new Sphere(0.4f);
		
		Color3f white = new Color3f(1, 1, 1);
		Color3f black = new Color3f(0, 0, 0);
		Color3f blue = new Color3f(0, 0, 1);
		Material mat = new Material(blue, black, white, white, 0.6f);
		Appearance app = new Appearance();
		app.setMaterial(mat);
		ball.setAppearance(app);

		AmbientLight light1 = new AmbientLight(new Color3f(1,1,1));
		BoundingSphere bounds = new BoundingSphere(new Point3d(), 100);
		light1.setInfluencingBounds(bounds);			//setzt die Reichweite der Lichtquelle
		bg.addChild(light1);

		DirectionalLight light2 = new DirectionalLight(
				new Color3f(.5f, .5f, .5f),
				new Vector3f(1, 0, -1));
		light2.setInfluencingBounds(bounds);			//setzt die Reichweite der Lichtquelle
		bg.addChild(light2);

		Transform3D verschiebung = new Transform3D();
		verschiebung.setTranslation(new Vector3f(3, 3, -10));
	
		TransformGroup tg = new TransformGroup(verschiebung);
		tg.addChild(ball);
		bg.addChild(tg);

		return bg;
	}
	
	/**
	 * Erzeugt eine blaue Kugel, die sich in zwei Sekunden einmal um die eigene
	 * Achse dreht.
	 */
	private BranchGroup buildAnimationVersion(){
		BranchGroup bg = new BranchGroup();
		Sphere ball = new Sphere(0.4f);
		
		Color3f white = new Color3f(1, 1, 1);
		Color3f black = new Color3f(0, 0, 0);
		Color3f blue = new Color3f(0, 0, 1);
		Material mat = new Material(blue, black, white, white, 0.6f);
		Appearance app = new Appearance();
		app.setMaterial(mat);
		ball.setAppearance(app);

		AmbientLight light1 = new AmbientLight(new Color3f(1,1,1));
		BoundingSphere bounds = new BoundingSphere(new Point3d(), 100);
		light1.setInfluencingBounds(bounds);
		bg.addChild(light1);

		DirectionalLight light2 = new DirectionalLight(
				new Color3f(.5f, .5f, .5f),
				new Vector3f(1, 0, -1));
		light2.setInfluencingBounds(bounds);
		bg.addChild(light2);

		TransformGroup tg = new TransformGroup();
		tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		Alpha rotationAlpha = new Alpha(-1, 2000);
		RotationInterpolator rotator = 
			new RotationInterpolator(rotationAlpha, tg);
		rotator.setSchedulingBounds(bounds);
		tg.addChild(rotator);

		tg.addChild(ball);
		bg.addChild(tg);

		return bg;		
	}
	
	/**
	 * Lsst den Benutzer die Position der blauen Kugel mithilfe der
	 * Pfeiltasten (X- und Y-Koordinaten) und '0' und '9' (Z-Koordinate)
	 * verndern.
	 * @param c - Canvas3D-Objekt. Wird bentigt, um die Tastatureingaben abzufangen.
	 */
	private BranchGroup buildKeyboardInteractionVersion(Canvas3D c){
		BranchGroup bg = new BranchGroup();
		Sphere ball = new Sphere(0.4f);
		
		Color3f white = new Color3f(1,1,1);
		Color3f black = new Color3f(0,0,0);
		Color3f blue = new Color3f(0, 0, 1);
		Material mat = new Material(blue, black, white, white, 0.6f);
		
		Appearance app = new Appearance();
		app.setMaterial(mat);
		ball.setAppearance(app);
		
		AmbientLight light1 = new AmbientLight(new Color3f(1,1,1));
		BoundingSphere bounds = new BoundingSphere(new Point3d(), 100);
		light1.setInfluencingBounds(bounds);
		bg.addChild(light1);
		
		DirectionalLight light2 = new DirectionalLight(new Color3f(.5f, .5f, .5f), new Vector3f(3, 3, -10));
		light2.setInfluencingBounds(bounds);
		bg.addChild(light2);
		
		tg = new TransformGroup();
		tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		position = new Vector3f();
		
		c.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent e){
				switch(e.getKeyCode()){
				case KeyEvent.VK_LEFT:
					position.x -= 0.1f;
					break;
					case KeyEvent.VK_RIGHT:
						position.x += 0.1f;
						break;
					case KeyEvent.VK_UP:
						position.y += 0.1f;
						break;
					case KeyEvent.VK_DOWN:
						position.y -= 0.1f;
						break;
					case KeyEvent.VK_0:
						position.z -= 0.1f;
						break;
					case KeyEvent.VK_9:
						position.z += 0.1f;
						break;
				}
				Transform3D t3d = new Transform3D();
				t3d.setTranslation(position);
				tg.setTransform(t3d);
			}
		});
		
		
		tg.addChild(ball);
		bg.addChild(tg);
		
		return bg;
	}
	
	public static void main(String[] args){
		HelloUniverse hello = new HelloUniverse();
		hello.setSize(800, 600);
		hello.setVisible(true);
	}
}
